@@ -13,38 +13,41 @@
 import Command from '@oclif/command'
 import { cli } from 'cli-ux'
 import * as yaml from 'js-yaml'
-import Listr = require('listr')
 import * as path from 'path'
-import { OLM, OLMInstallationUpdate } from '../../api/context'
 import { CheHelper } from '../../api/che'
+import { OLM, OLMInstallationUpdate } from '../../api/context'
 import { KubeHelper } from '../../api/kube'
 import { CatalogSource, Subscription } from '../../api/types/olm'
 import { VersionHelper } from '../../api/version'
-import { CUSTOM_CATALOG_SOURCE_NAME, CVS_PREFIX, DEFAULT_CHE_NAMESPACE, DEFAULT_CHE_OLM_PACKAGE_NAME, DEFAULT_OLM_KUBERNETES_NAMESPACE, DEFAULT_OPENSHIFT_MARKET_PLACE_NAMESPACE, DEFAULT_OPENSHIFT_OPERATORS_NS_NAME, KUBERNETES_OLM_CATALOG, NEXT_CATALOG_SOURCE_NAME, OLM_NEXT_CHANNEL_NAME, OLM_STABLE_CHANNEL_NAME, OPENSHIFT_OLM_CATALOG, OPERATOR_GROUP_NAME, OLM_STABLE_ALL_NAMESPACES_CHANNEL_NAME, DEFAULT_CHE_OPERATOR_SUBSCRIPTION_NAME, OLM_NEXT_ALL_NAMESPACES_CHANNEL_NAME, OLM_STABLE_CHANNEL_STARTING_CSV_TEMPLATE, OLM_STABLE_ALL_NAMESPACES_CHANNEL_STARTING_CSV_TEMPLATE } from '../../constants'
-import { getEmbeddedTemplatesDirectory, isKubernetesPlatformFamily } from '../../util'
-
+import { CHECTL_PROJECT_NAME, CSV_PREFIX, CUSTOM_CATALOG_SOURCE_NAME, DEFAULT_CHE_NAMESPACE, DEFAULT_CHE_OLM_PACKAGE_NAME, DEFAULT_CHE_OPERATOR_SUBSCRIPTION_NAME, DEFAULT_OLM_KUBERNETES_NAMESPACE, DEFAULT_OPENSHIFT_MARKET_PLACE_NAMESPACE, DEFAULT_OPENSHIFT_OPERATORS_NS_NAME, INDEX_IMG, KUBERNETES_OLM_CATALOG, NEXT_CATALOG_SOURCE_NAME, OLM_NEXT_ALL_NAMESPACES_CHANNEL_NAME, OLM_NEXT_CHANNEL_NAME, OLM_STABLE_ALL_NAMESPACES_CHANNEL_NAME, OLM_STABLE_ALL_NAMESPACES_CHANNEL_STARTING_CSV_TEMPLATE, OLM_STABLE_CHANNEL_NAME, OLM_STABLE_CHANNEL_STARTING_CSV_TEMPLATE, OPENSHIFT_OLM_CATALOG, OPERATOR_GROUP_NAME } from '../../constants'
+import { getEmbeddedTemplatesDirectory, getProjectName, isKubernetesPlatformFamily } from '../../util'
 import { createEclipseCheCluster, patchingEclipseCheCluster } from './common-tasks'
+import { OLMDevWorkspaceTasks } from './olm-dev-workspace-operator'
+import Listr = require('listr')
 
 export const TASK_TITLE_SET_CUSTOM_OPERATOR_IMAGE = 'Set custom operator image'
 export const TASK_TITLE_CREATE_CUSTOM_CATALOG_SOURCE_FROM_FILE = 'Create custom catalog source from file'
 export const TASK_TITLE_PREPARE_CHE_CLUSTER_CR = 'Prepare CodeReady Workspaces cluster CR'
 
 export const TASK_TITLE_DELETE_CUSTOM_CATALOG_SOURCE = `Delete(OLM) custom catalog source ${CUSTOM_CATALOG_SOURCE_NAME}`
-export const TASK_TITLE_DELETE_NIGHTLY_CATALOG_SOURCE = `Delete(OLM) nigthly catalog source ${NEXT_CATALOG_SOURCE_NAME}`
+export const TASK_TITLE_DELETE_NEXT_CATALOG_SOURCE = `Delete(OLM) nigthly catalog source ${NEXT_CATALOG_SOURCE_NAME}`
 
 export class OLMTasks {
-  prometheusRoleName = 'prometheus-k8s'
-  prometheusRoleBindingName = 'prometheus-k8s'
+  private readonly prometheusRoleName = 'prometheus-k8s'
+  private readonly prometheusRoleBindingName = 'prometheus-k8s'
+  private readonly kube: KubeHelper
+  private readonly che: CheHelper
+  private readonly olmDevWorkspaceTasks: OLMDevWorkspaceTasks
+
+  constructor(flags: any) {
+    this.kube = new KubeHelper(flags)
+    this.che = new CheHelper(flags)
+    this.olmDevWorkspaceTasks = new OLMDevWorkspaceTasks(flags)
+  }
 
-  /**
-   * Returns list of tasks which perform preflight platform checks.
-   */
   startTasks(flags: any, command: Command): Listr.ListrTask<any>[] {
-    const kube = new KubeHelper(flags)
-    const che = new CheHelper(flags)
-
     return [
-      this.isOlmPreInstalledTask(command, kube),
+      this.isOlmPreInstalledTask(command),
       {
         title: 'Configure context information',
         task: async (ctx: any, task: any) => {
@@ -92,11 +95,11 @@ export class OLMTasks {
         enabled: () => flags['cluster-monitoring'] && flags.platform === 'openshift',
         title: `Create Role ${this.prometheusRoleName} in namespace ${flags.chenamespace}`,
         task: async (_ctx: any, task: any) => {
-          if (await kube.isRoleExist(this.prometheusRoleName, flags.chenamespace)) {
+          if (await this.kube.isRoleExist(this.prometheusRoleName, flags.chenamespace)) {
             task.title = `${task.title}...[Exists]`
           } else {
             const yamlFilePath = path.join(getEmbeddedTemplatesDirectory(), '..', 'resources', 'prometheus-role.yaml')
-            await kube.createRoleFromFile(yamlFilePath, flags.chenamespace)
+            await this.kube.createRoleFromFile(yamlFilePath, flags.chenamespace)
             task.title = `${task.title}...[OK].`
           }
         },
@@ -105,53 +108,56 @@ export class OLMTasks {
         enabled: () => flags['cluster-monitoring'] && flags.platform === 'openshift',
         title: `Create RoleBinding ${this.prometheusRoleBindingName} in namespace ${flags.chenamespace}`,
         task: async (_ctx: any, task: any) => {
-          if (await kube.isRoleBindingExist(this.prometheusRoleBindingName, flags.chenamespace)) {
+          if (await this.kube.isRoleBindingExist(this.prometheusRoleBindingName, flags.chenamespace)) {
             task.title = `${task.title}...[Exists]`
           } else {
             const yamlFilePath = path.join(getEmbeddedTemplatesDirectory(), '..', 'resources', 'prometheus-role-binding.yaml')
-            await kube.createRoleBindingFromFile(yamlFilePath, flags.chenamespace)
+            await this.kube.createRoleBindingFromFile(yamlFilePath, flags.chenamespace)
             task.title = `${task.title}...[OK]`
           }
         },
       },
       {
         title: 'Create operator group',
-        // 'stable-all-namespaces' and 'next-all-namespaces' channels install the operator in openshift-operators namespace and there already exists a pre-created operator-group.
         enabled: (ctx: any) => ctx.operatorNamespace !== DEFAULT_OPENSHIFT_OPERATORS_NS_NAME,
         task: async (_ctx: any, task: any) => {
-          if (await che.findCheOperatorOperatorGroup(flags.chenamespace)) {
+          if (await this.che.findCheOperatorOperatorGroup(flags.chenamespace)) {
             task.title = `${task.title}...[Exists]`
           } else {
-            await kube.createOperatorGroup(OPERATOR_GROUP_NAME, flags.chenamespace)
+            await this.kube.createOperatorGroup(OPERATOR_GROUP_NAME, flags.chenamespace)
             task.title = `${task.title}...[OK]`
           }
         },
       },
       {
-        enabled: () => !VersionHelper.isDeployingStableVersion(flags) && !flags[OLM.CATALOG_SOURCE_NAME] && !flags[OLM.CATALOG_SOURCE_YAML] && flags[OLM.CHANNEL] !== OLM_STABLE_CHANNEL_NAME,
-        title: 'Create next index CatalogSource',
+        enabled: () => !flags[OLM.CATALOG_SOURCE_NAME] && !flags[OLM.CATALOG_SOURCE_YAML] && flags[OLM.CHANNEL] !== OLM_STABLE_CHANNEL_NAME,
+        title: 'Create CatalogSource for \'next\' channel',
         task: async (ctx: any, task: any) => {
-          if (!await kube.IsCatalogSourceExists(NEXT_CATALOG_SOURCE_NAME, ctx.operatorNamespace)) {
-            const catalogSourceImage = `quay.io/eclipse/eclipse-che-${ctx.generalPlatformName}-opm-catalog:next`
-            const nextCatalogSource = this.constructIndexCatalogSource(ctx.operatorNamespace, catalogSourceImage)
-            await kube.createCatalogSource(nextCatalogSource)
-            await kube.waitCatalogSource(ctx.operatorNamespace, NEXT_CATALOG_SOURCE_NAME)
+          if (!await this.kube.IsCatalogSourceExists(NEXT_CATALOG_SOURCE_NAME, ctx.operatorNamespace)) {
+            const nextCatalogSource = this.constructNextCatalogSource(ctx.operatorNamespace)
+            await this.kube.createCatalogSource(nextCatalogSource)
+            await this.kube.waitCatalogSource(ctx.operatorNamespace, NEXT_CATALOG_SOURCE_NAME)
             task.title = `${task.title}...[OK]`
           } else {
             task.title = `${task.title}...[Exists]`
           }
         },
       },
+      {
+        enabled: () => getProjectName() === CHECTL_PROJECT_NAME,
+        title: 'Deploy Dev Workspace operator',
+        task: async () => this.olmDevWorkspaceTasks.startTasks(flags, command),
+      },
       {
         title: TASK_TITLE_CREATE_CUSTOM_CATALOG_SOURCE_FROM_FILE,
         enabled: () => flags[OLM.CATALOG_SOURCE_YAML],
         task: async (ctx: any, task: any) => {
-          const customCatalogSource: CatalogSource = kube.readCatalogSourceFromFile(flags[OLM.CATALOG_SOURCE_YAML])
-          if (!await kube.IsCatalogSourceExists(customCatalogSource.metadata!.name!, flags.chenamespace)) {
+          const customCatalogSource: CatalogSource = this.kube.readCatalogSourceFromFile(flags[OLM.CATALOG_SOURCE_YAML])
+          if (!await this.kube.IsCatalogSourceExists(customCatalogSource.metadata!.name!, flags.chenamespace)) {
             customCatalogSource.metadata.name = ctx.sourceName
             customCatalogSource.metadata.namespace = flags.chenamespace
-            await kube.createCatalogSource(customCatalogSource)
-            await kube.waitCatalogSource(flags.chenamespace, CUSTOM_CATALOG_SOURCE_NAME)
+            await this.kube.createCatalogSource(customCatalogSource)
+            await this.kube.waitCatalogSource(flags.chenamespace, CUSTOM_CATALOG_SOURCE_NAME)
             task.title = `${task.title}...[OK: ${CUSTOM_CATALOG_SOURCE_NAME}]`
           } else {
             task.title = `${task.title}...[Exists]`
@@ -161,7 +167,7 @@ export class OLMTasks {
       {
         title: 'Create operator subscription',
         task: async (ctx: any, task: any) => {
-          let subscription = await che.findCheOperatorSubscription(ctx.operatorNamespace)
+          let subscription = await this.che.findCheOperatorSubscription(ctx.operatorNamespace)
           if (subscription) {
             ctx.subscriptionName = subscription.metadata.name
             task.title = `${task.title}...[Exists]`
@@ -183,14 +189,14 @@ export class OLMTasks {
           } else {
             throw new Error(`Unknown OLM channel ${flags[OLM.CHANNEL]}`)
           }
-          await kube.createOperatorSubscription(subscription)
+          await this.kube.createOperatorSubscription(subscription)
           task.title = `${task.title}...[OK]`
         },
       },
       {
         title: 'Wait while subscription is ready',
         task: async (ctx: any, task: any) => {
-          const installPlan = await kube.waitOperatorSubscriptionReadyForApproval(ctx.operatorNamespace, ctx.subscriptionName, 600)
+          const installPlan = await this.kube.waitOperatorSubscriptionReadyForApproval(ctx.operatorNamespace, ctx.subscriptionName, 600)
           ctx.installPlanName = installPlan.name
           task.title = `${task.title}...[OK]`
         },
@@ -199,7 +205,28 @@ export class OLMTasks {
         title: 'Approve installation',
         enabled: ctx => ctx.approvalStrategy === OLMInstallationUpdate.MANUAL,
         task: async (ctx: any, task: any) => {
-          await kube.approveOperatorInstallationPlan(ctx.installPlanName, ctx.operatorNamespace)
+          await this.kube.approveOperatorInstallationPlan(ctx.installPlanName, ctx.operatorNamespace)
+          task.title = `${task.title}...[OK]`
+        },
+      },
+      {
+        title: 'Wait operator install plan',
+        task: async (ctx: any, task: any) => {
+          await this.kube.waitOperatorInstallPlan(ctx.installPlanName, ctx.operatorNamespace)
+          task.title = `${task.title}...[OK]`
+        },
+      },
+      {
+        title: 'Check cluster service version resource',
+        task: async (ctx: any, task: any) => {
+          const installedCSV = await this.kube.waitInstalledCSV(ctx.operatorNamespace, ctx.subscriptionName)
+          const csv = await this.kube.getCSV(installedCSV, ctx.operatorNamespace)
+          if (!csv) {
+            throw new Error(`cluster service version resource ${installedCSV} not found`)
+          }
+          if (csv.status.phase === 'Failed') {
+            throw new Error(`cluster service version resource failed. Cause: ${csv.status.message}. Reason: ${csv.status.reason}.`)
+          }
           task.title = `${task.title}...[OK]`
         },
       },
@@ -207,59 +234,89 @@ export class OLMTasks {
         title: TASK_TITLE_SET_CUSTOM_OPERATOR_IMAGE,
         enabled: () => flags['che-operator-image'],
         task: async (_ctx: any, task: any) => {
-          const csvList = await kube.getClusterServiceVersions(flags.chenamespace)
+          const csvList = await this.kube.getClusterServiceVersions(flags.chenamespace)
           if (csvList.items.length < 1) {
             throw new Error('Failed to get CSV for Che operator')
           }
           const csv = csvList.items[0]
           const jsonPatch = [{ op: 'replace', path: '/spec/install/spec/deployments/0/spec/template/spec/containers/0/image', value: flags['che-operator-image'] }]
-          await kube.patchClusterServiceVersion(csv.metadata.namespace!, csv.metadata.name!, jsonPatch)
-          task.title = `${task.title}...[OK]`
-        },
-      },
-      {
-        title: 'Wait while operator installed',
-        task: async (ctx: any, task: any) => {
-          await kube.waitUntilOperatorIsInstalled(ctx.installPlanName, ctx.operatorNamespace)
+          await this.kube.patchClusterServiceVersion(csv.metadata.namespace!, csv.metadata.name!, jsonPatch)
           task.title = `${task.title}...[OK]`
         },
       },
       {
         title: TASK_TITLE_PREPARE_CHE_CLUSTER_CR,
         task: async (ctx: any, task: any) => {
-          const cheCluster = await kube.getCheCluster(flags.chenamespace)
+          const cheCluster = await this.kube.getCheCluster(flags.chenamespace)
           if (cheCluster) {
             task.title = `${task.title}...[Exists]`
             return
           }
 
           if (!ctx.customCR) {
-            ctx.defaultCR = await this.getCRFromCSV(kube, ctx.operatorNamespace, ctx.subscriptionName)
+            ctx.defaultCR = await this.getCRFromCSV(ctx.operatorNamespace, ctx.subscriptionName)
           }
 
           task.title = `${task.title}...[OK]`
         },
       },
-      createEclipseCheCluster(flags, kube),
+      createEclipseCheCluster(flags, this.kube),
     ]
   }
 
   preUpdateTasks(flags: any, command: Command): Listr {
-    const kube = new KubeHelper(flags)
-    const che = new CheHelper(flags)
     return new Listr([
-      this.isOlmPreInstalledTask(command, kube),
+      this.isOlmPreInstalledTask(command),
       {
-        title: 'Check if operator subscription exists',
-        task: async (ctx: any, task: any) => {
-          const subscription = await che.findCheOperatorSubscription(flags.chenamespace)
+        title: 'Check operator subscription',
+        task: async (ctx: any, task: Listr.ListrTaskWrapper<any>) => {
+          const subscription = await this.che.findCheOperatorSubscription(flags.chenamespace)
           if (!subscription) {
             command.error('Unable to find operator subscription')
           }
           ctx.operatorNamespace = subscription.metadata.namespace
+          ctx.installPlanApproval = subscription.spec.installPlanApproval
+
+          if (subscription.spec.installPlanApproval === OLMInstallationUpdate.AUTO) {
+            task.title = `${task.title}...[Interrupted]`
+            return new Listr([
+              {
+                title: '[Warning] OLM itself manage operator update with installation mode \'Automatic\'.',
+                task: () => { },
+              },
+              {
+                title: '[Warning] Use \'crwctl server:update\' command only with \'Manual\' installation plan approval.',
+                task: () => {
+                  command.exit(0)
+                },
+              },
+            ], ctx.listrOptions)
+          }
+
           task.title = `${task.title}...[OK]`
         },
       },
+      {
+        title: 'Check if CheCluster CR exists',
+        task: async (ctx: any, _task: any) => {
+          if (ctx.operatorNamespace === DEFAULT_OPENSHIFT_OPERATORS_NS_NAME) {
+            const cheClusters = await this.kube.getAllCheClusters()
+            if (cheClusters.length === 0) {
+              command.error(`CodeReady Workspaces cluster CR was not found in the namespace '${flags.chenamespace}'`)
+            }
+            if (cheClusters.length > 0) {
+              command.error('CodeReady Workspaces does not support more than one installation in all namespaces mode.')
+            }
+            ctx.checlusterNamespace = cheClusters[0].metadata.namespace
+          } else {
+            const cheCluster = await this.kube.getCheCluster(ctx.operatorNamespace)
+            if (!cheCluster) {
+              command.error(`CodeReady Workspaces cluster CR was not found in the namespace '${flags.chenamespace}'`)
+            }
+            ctx.checlusterNamespace = cheCluster.metadata.namespace
+          }
+        },
+      },
     ], { renderer: flags['listr-renderer'] as any })
   }
 
@@ -315,7 +372,7 @@ export class OLMTasks {
         title: 'Wait while newer operator installed',
         enabled: (ctx: any) => ctx.installPlanName,
         task: async (ctx: any, task: any) => {
-          await kube.waitUntilOperatorIsInstalled(ctx.installPlanName, ctx.operatorNamespace, 60)
+          await kube.waitOperatorInstallPlan(ctx.installPlanName, ctx.operatorNamespace, 60)
           ctx.highlightedMessages.push(`Operator is updated from ${ctx.currentVersion} to ${ctx.nextVersion} version`)
           task.title = `${task.title}...[OK]`
         },
@@ -352,7 +409,7 @@ export class OLMTasks {
         },
       },
       {
-        title: 'Delete(OLM) operator subscription',
+        title: 'Delete operator subscription',
         enabled: ctx => ctx.isPreInstalledOLM && ctx.subscriptionName,
         task: async (ctx: any, task: any) => {
           await kube.deleteOperatorSubscription(ctx.subscriptionName, ctx.operatorNamespace)
@@ -360,11 +417,11 @@ export class OLMTasks {
         },
       },
       {
-        title: 'Delete(OLM) CodeReady Workspaces cluster service versions',
+        title: 'Delete CodeReady Workspaces cluster service versions',
         enabled: ctx => ctx.isPreInstalledOLM,
         task: async (ctx: any, task: any) => {
           const csvs = await kube.getClusterServiceVersions(ctx.operatorNamespace)
-          const csvsToDelete = csvs.items.filter(csv => csv.metadata.name!.startsWith(CVS_PREFIX))
+          const csvsToDelete = csvs.items.filter(csv => csv.metadata.name!.startsWith(CSV_PREFIX))
           for (const csv of csvsToDelete) {
             await kube.deleteClusterServiceVersion(ctx.operatorNamespace, csv.metadata.name!)
           }
@@ -372,7 +429,7 @@ export class OLMTasks {
         },
       },
       {
-        title: 'Delete(OLM) operator group',
+        title: 'Delete operator group',
         // Do not delete global operator group if operator is in all namespaces mode
         enabled: ctx => ctx.isPreInstalledOLM && ctx.operatorNamespace !== DEFAULT_OPENSHIFT_OPERATORS_NS_NAME,
         task: async (ctx: any, task: any) => {
@@ -391,7 +448,7 @@ export class OLMTasks {
         },
       },
       {
-        title: TASK_TITLE_DELETE_NIGHTLY_CATALOG_SOURCE,
+        title: TASK_TITLE_DELETE_NEXT_CATALOG_SOURCE,
         task: async (ctx: any, task: any) => {
           await kube.deleteCatalogSource(ctx.operatorNamespace, NEXT_CATALOG_SOURCE_NAME)
           task.title = `${task.title}...[OK]`
@@ -414,11 +471,11 @@ export class OLMTasks {
     ]
   }
 
-  private isOlmPreInstalledTask(command: Command, kube: KubeHelper): Listr.ListrTask<Listr.ListrContext> {
+  private isOlmPreInstalledTask(command: Command): Listr.ListrTask<Listr.ListrContext> {
     return {
       title: 'Check if OLM is pre-installed on the platform',
       task: async (_ctx: any, task: any) => {
-        if (!await kube.isPreInstalledOLM()) {
+        if (!await this.kube.isPreInstalledOLM()) {
           cli.warn('Looks like your platform hasn\'t got embedded OLM, so you should install it manually. For quick start you can use:')
           cli.url('install.sh', 'https://raw.githubusercontent.com/operator-framework/operator-lifecycle-manager/master/deploy/upstream/quickstart/install.sh')
           command.error('OLM is required for installation of CodeReady Workspaces with installer flag \'olm\'')
@@ -447,7 +504,7 @@ export class OLMTasks {
     }
   }
 
-  private constructIndexCatalogSource(namespace: string, catalogSourceImage: string): CatalogSource {
+  private constructNextCatalogSource(namespace: string): CatalogSource {
     return {
       apiVersion: 'operators.coreos.com/v1alpha1',
       kind: 'CatalogSource',
@@ -456,7 +513,7 @@ export class OLMTasks {
         namespace,
       },
       spec: {
-        image: catalogSourceImage,
+        image: INDEX_IMG,
         sourceType: 'grpc',
         updateStrategy: {
           registryPoll: {
@@ -467,18 +524,19 @@ export class OLMTasks {
     }
   }
 
-  private async getCRFromCSV(kube: KubeHelper, namespace: string, subscriptionName: string): Promise<any> {
-    const subscription = await kube.getOperatorSubscription(subscriptionName, namespace)
+  private async getCRFromCSV(namespace: string, subscriptionName: string): Promise<any> {
+    const subscription = await this.kube.getOperatorSubscription(subscriptionName, namespace)
     if (!subscription) {
       throw new Error(`Subscription '${subscriptionName}' not found in namespace '${namespace}'`)
     }
     const installedCSV = subscription.status!.installedCSV!
-    const csv = await kube.getCSV(installedCSV, namespace)
+    const csv = await this.kube.getCSV(installedCSV, namespace)
+
     if (csv && csv.metadata.annotations) {
       const CRRaw = csv.metadata.annotations!['alm-examples']
       return (yaml.load(CRRaw) as Array<any>).find(cr => cr.kind === 'CheCluster')
     } else {
-      throw new Error(`Unable to retrieve Che cluster CR definition from CSV: ${installedCSV}`)
+      throw new Error(`Unable to retrieve CheCluster CR ${!csv ? '' : 'from CSV: ' + csv.spec.displayName}`)
     }
   }
 }
